@@ -10,7 +10,7 @@ from tornado import stack_context
 from tornado.ioloop import IOLoop
 
 
-version_tuple = (0, 2)
+version_tuple = (0, 3)
 
 version = '.'.join(map(str, version_tuple))
 """Current version of Toro."""
@@ -337,14 +337,8 @@ class Queue(ToroBase):
 
     :doc:`examples/web_spider_example`
 
-    .. warning:: Although the ``maxsize`` attribute is mutable, increasing it
-      does not automatically unblock functions waiting to :meth:`put <Queue.put>`
-      items. This is a bug.
-
-    .. todo:: Fix it.
-
     :Parameters:
-      - `max_size`: Optional size limit (no limit by default).
+      - `maxsize`: Optional size limit (no limit by default).
       - `initial`: Optional sequence of initial items.
       - `io_loop`: Optional custom IOLoop.
 
@@ -356,7 +350,7 @@ class Queue(ToroBase):
         super(Queue, self).__init__(io_loop)
         if maxsize is not None and maxsize < 0:
             raise ValueError("maxsize can't be negative")
-        self.maxsize = maxsize
+        self._maxsize = maxsize
 
         # _Waiters
         self.getters = collections.deque([])
@@ -398,6 +392,42 @@ class Queue(ToroBase):
         """Number of items in the queue"""
         return len(self.queue)
 
+    def get_maxsize(self):
+        """Number of items allowed in the queue.
+
+        Setting maxsize to a larger number unblocks callbacks waiting
+        on :meth:`put`. Decreasing maxsize below :meth:`qsize`
+        raises :exc:`RuntimeError`:
+
+        >>> from tornado import gen
+        >>> q = toro.Queue(0)
+        >>> @gen.engine
+        ... def f():
+        ...     print 'Trying to put...'
+        ...     yield gen.Task(q.put, 'a')
+        ...     print 'Done putting'
+        ...
+        >>> f()
+        Trying to put...
+        >>> q.maxsize = 1
+        Done putting
+        """
+        return self._maxsize
+
+    def set_maxsize(self, maxsize):
+        if maxsize < self.qsize():
+            raise RuntimeError("%s has %d items, can't set maxsize to %d" % (
+                repr(self), self.qsize(), maxsize))
+
+        self._maxsize = maxsize
+        while self.qsize() < maxsize and self.putters:
+            item, putter = self.putters.popleft()
+            if not putter.expired:
+                self._put(item)
+                putter.run(True)
+
+    maxsize = property(get_maxsize, set_maxsize)
+
     def empty(self):
         """Return ``True`` if the queue is empty, ``False`` otherwise."""
         return not self.queue
@@ -450,16 +480,7 @@ class Queue(ToroBase):
             self._run_callback(callback, True)
         elif self.maxsize == self.qsize() and callback:
             _check_callback(callback)
-
-            # When a getter runs and frees up a slot so this putter can run,
-            # we need to defer the put callback for one more iteration of the
-            # loop to ensure that getters and putters alternate perfectly.
-            # See TestChannel2.test_wait.
-            io_loop = self.io_loop
-            def _callback(success):
-                io_loop.add_callback(partial(callback, success))
-
-            waiter = _Waiter(deadline, (Full,), self.io_loop, _callback)
+            waiter = _Waiter(deadline, (Full,), self.io_loop, callback)
             self.putters.append((item, waiter))
         elif self.maxsize == self.qsize():
             raise Full
@@ -494,7 +515,11 @@ class Queue(ToroBase):
             self._put(item)
             if callback:
                 self._run_callback(callback, self._get())
-                putter.run(True)
+                # When a getter runs and frees up a slot so this putter can
+                # run, we need to defer the put callback for an iteration
+                # of the loop to ensure that getters and putters alternate
+                # perfectly. See TestChannel2.test_wait.
+                self.io_loop.add_callback(partial(putter.run, True))
             else:
                 self.io_loop.add_callback(partial(putter.run, True))
                 return self._get()
@@ -517,7 +542,7 @@ class PriorityQueue(Queue):
     Entries are typically tuples of the form: ``(priority number, data)``.
 
     :Parameters:
-      - `max_size`: Optional size limit (no limit by default).
+      - `maxsize`: Optional size limit (no limit by default).
       - `initial`: Optional sequence of initial items.
       - `io_loop`: Optional custom IOLoop.
     """
@@ -536,7 +561,7 @@ class LifoQueue(Queue):
     first.
 
     :Parameters:
-      - `max_size`: Optional size limit (no limit by default).
+      - `maxsize`: Optional size limit (no limit by default).
       - `initial`: Optional sequence of initial items.
       - `io_loop`: Optional custom IOLoop.
     """
@@ -557,7 +582,7 @@ class JoinableQueue(Queue):
     .. seealso:: :doc:`examples/web_spider_example`
 
     :Parameters:
-      - `max_size`: Optional size limit (no limit by default).
+      - `maxsize`: Optional size limit (no limit by default).
       - `initial`: Optional sequence of initial items.
       - `io_loop`: Optional custom IOLoop.
     """
